shapiro_table_function <- function(data,num_col){
  
  shapiro <- data.frame()
  
  
  for (col in num_col){
    tryCatch({
      a <- shapiro.test(as.numeric(data[[col]]))
      shapiro <- rbind(shapiro,c(col,"","Shapiro",a$p.value))
      colnames(shapiro) <- c("Column","Classe","Test","p-value")
    },
    error=function(e) {
      print(co)
      message('An Error Occurred')
      cat("Erreur :", conditionMessage(e), "\n")
      traceback()
    },
    warning=function(w) {
      message('A Warning Occured')
      print(w)
      return(NA)
    })
    
  }
  
  return(shapiro)
}




anova_table_function <- function(data,var_expliquee,var_explicatives,output) {
  tryCatch({
    form = as.formula(paste0(paste0(var_expliquee,"~"),paste(var_explicatives,collapse = "*")))
    mod = lm(as.formula(form),data=data)
    anova_table <- as.data.frame(Anova(mod))
    for (col in names(anova_table)) {
      if (is.numeric(anova_table[[col]])) {
        anova_table[[col]] <- round(anova_table[[col]], 3)
      }
    }
    output$anova_result <- renderPrint({return(Anova(mod))})
    return(anova_table)
  },
  error = function(e) {
    message('An Error Occurred')
    cat("Erreur :", conditionMessage(e), "\n")
    traceback()
    return(NULL)
  },
  warning = function(w) {
    message('A Warning Occured')
    print(w)
    return(a)
  })
  
}

# Pour le modèle mixte : summary(anova) R2 graphique de diag + test comparaison modèles et test de comparaison
emmeans_table_function <- function(data, var_expliquee, var_explicatives, type_comp,output,ref_index) {
  tryCatch({
    # Créer les formules
    if (type_comp == 'trt.vs.ctrl') {
      structure_formule = as.formula(paste0("trt.vs.ctrl~",paste(var_explicatives,collapse = "*")))
    } else {
      structure_formule = as.formula(paste0("pairwise~",paste(var_explicatives,collapse = "*")))
    }
    
    formule_regression <- as.formula(paste0(var_expliquee, "~", paste(var_explicatives, collapse = "*")))
    
    # Ajuster le modèle de régression linéaire
    mod <- lm(formule_regression, data = data)
    
    # Calculer les moyennes marginales estimées (emmeans)
    emm <- emmeans(mod, structure_formule)
    
    # Calculer les contrastes
    contr <- contrast(emm, type_comp, ref = ref_index)
    
    # Convertir en tableau de données et arrondir
    emmeans_table <- as.data.frame(contr)
    
    for (col in names(emmeans_table)) {
      if (is.numeric(emmeans_table[[col]])) {
        emmeans_table[[col]] <- round(emmeans_table[[col]], 3)
      }
    }
    
    emmeans_table <- na.omit(emmeans_table)
    output$graph_compared_mean <- renderPlotly({
      return(comparisonGraph(emmeansObj = emm))
    })
    output$emmeans_result <- renderPrint({
      return(emm)
    })
    return(emmeans_table)
    
  }, warning = function(w) {
    warning("Avertissement rencontré : ", conditionMessage(w))
    return(NULL)
  }, error = function(e) {
    message("Erreur rencontrée : ", conditionMessage(e))
    return(NULL)
  })
}


contrasts_table_function <- function(data, var_expliquee, var_explicatives, type_comp, output,ref_index) {
  tryCatch({
    # Créer les formules
    structure_formule <- as.formula(paste0("~",paste(var_explicatives,collapse = "*")))
    formule_regression <- as.formula(paste0(paste0(var_expliquee,"~"),paste(var_explicatives,collapse = "*")))
    
    # Ajuster le modèle de régression linéaire
    mod <- lm(formule_regression, data = data)
    
    # Calculer les moyennes marginales estimées (emmeans)
    emm <- emmeans(mod, structure_formule)
    
    # Calculer les contrastes
    contr <- contrast(emm, type_comp, ref = ref_index)
    
    # Convertir en tableau de données et arrondir
    emmeans_table <- as.data.frame(contr)
    
    for (col in names(emmeans_table)) {
      if (is.numeric(emmeans_table[[col]])) {
        emmeans_table[[col]] <- round(emmeans_table[[col]], 3)
      }
    }
    
    emmeans_table <- na.omit(emmeans_table)
    return(emmeans_table)
    
  }, warning = function(w) {
    warning("Avertissement rencontré : ", conditionMessage(w))
    return(NULL)
  }, error = function(e) {
    message("Erreur rencontrée : ", conditionMessage(e))
    return(NULL)
  })
}


ref_index_function <- function(data,var_expliquee,var_explicatives,input,Temoins){
  print(input)
  tryCatch({
    t = ""
    for (col in names(Temoins)) {
      t <- paste(t,input[[paste0("temoin_", col)]],collapse = " ")
    }
    print(t)
    t <- substr(t,2,nchar(t))
    form1 = as.formula(paste0("~",paste(var_explicatives,collapse = "*")))
    form = as.formula(paste0(paste0(var_expliquee,"~"),paste(var_explicatives,collapse = "*")))
    mod = lm(form,data=data)
    emm <- emmeans(mod,form1)
    row_concat <- apply(emm@grid[,-ncol(emm@grid)], 1, function(row) {
      paste(row, collapse = " ")
    })
    ind <- which(row_concat == t)
    return(ind)
  },
  error = function(e) {
    message('An Error Occurred')
    cat("Erreur :", conditionMessage(e), "\n")
    traceback()
    return(NULL)
  },
  warning = function(w) {
    message('A Warning Occured')
    print(w)
    formula <- formula(paste(num_column, "~", cat))
    a <- leveneTest(formula, data = merged_data)
    return(a)
  })
  
}

kw_table_function <- function(cat_col,num_col,data,output) {
  
  kw_table = data.frame()
  
  tryCatch({
    krustal_formule <- as.formula(paste0(num_col,'~',cat_col))
    
    kruskal_test <- kruskal.test(krustal_formule,data=data)
    kw_table <- rbind(kw_table,c(num_col,cat_col,"KW",kruskal_test$p.value))
    colnames(kw_table) <-c("Column","Classe","Test","p-value")
    output$kw_resultat <- renderPrint({
      return(kruskal_test)
    })
  },
  error=function(e) {
    message('An Error Occurred')
    cat("Erreur :", conditionMessage(e), "\n")
    traceback()
  },
  warning=function(w) {
    message('A Warning Occured')
    print(w)
    kruskal_test <- kruskal.test(merged_data()[[col]]~interaction(merged_data()[[cat]], merged_data()[[cat_id()]]))
    kw_table <- rbind(kw_table,c(col,cat,"KW",kruskal_test$p.value))
    colnames(kw_table) <-c("Column","Classe","Test","p-value")
  })

}