library(shinythemes)
library (DT)
library(ggplot2)
library(gridExtra)
library(emmeans)
library(readxl)
library(dplyr)
library(carData)
library(car)
library(factoextra)
library(shinylogs)
library(plotly)
library(auth0)
library(httr)
library(shinyjs)
library(openssl)
library(urltools)
library(openssl)
library(pls)
library(shinyBS)
library(multcomp)

source("modules/analyse_server.R")

server <- function(input, output, session) {
  track_usage(
    storage_mode = store_null()
  )
  
  # Partie permettant de fusionner les fichiers
  observe({
    
    merged_data <- eventReactive(input$files, {
      # On lit la liste des fichiers et on les fusionne
      # Charger les données à partir des fichiers sélectionnés
      tables <- lapply(input$files$datapath, function(datapath) {
        # Vérifier l'extension du fichier
        if (tools::file_ext(datapath) %in% c("csv", "CSV")) {
          data <- read.csv(datapath)  # Utilisez read.csv() pour les fichiers CSV
        } else if (tools::file_ext(datapath) %in% c("xlsx", "xls")) {
          data <- readxl::read_excel(datapath)  # Utilisez read_excel() pour les fichiers Excel
        } else {
          stop("Type de fichier non pris en charge : ", tools::file_ext(datapath))
        }
        
        print(data)
        filename <- input$files$name[input$files$datapath == datapath]  
        filename <- substr(filename, 1, nchar(filename) - 4)
        return(data)
      })
      
      
      concatenated_table <- bind_rows(tables)
      concatenated_table <- na.omit(concatenated_table)
      #concatenated_table <- concatenated_table[,colSums(is.na(concatenated_table))==0]
      return(concatenated_table)
    })
    
    columnTypes <- reactiveVal(list())
    
    observe({
      data <- merged_data()
      types <- list()
      for (col in setdiff(colnames(data),input$ignore_columns)) {
        column_class <- class(data[[col]])  # Utilisez class() pour obtenir la classe de la colonne
        default_choice <- switch(
          column_class,
          "Date" = "Date",
          "character" = "Character",
          "integer" = "Integer",
          "numeric" = "Numeric",
          "factor" = "Factor",
          "logical" = "Logical",
          "Date"  # Par défaut, utilisez "Date" si la classe n'est pas reconnue
        )
        types[[col]] <- selectInput(
          inputId = paste0("type_", col),
          label = col,
          choices = c("Date", "Character", "Numeric", "Factor", "Integer", "Logical"),
          selected = default_choice
        )
      }
      columnTypes(types)
    })
    
    output$columnTypes <- renderUI({
      types <- columnTypes()
      if (!is.null(types) && length(types) > 0) {
        div(
          lapply(names(types), function(col) {
            types[[col]]
          })
        )
      }
    })
    
    merged_data_type <- eventReactive(input$applyTypes, {
      data <- merged_data()[,setdiff(colnames(merged_data()),input$ignore_columns)]
      types <- columnTypes()
      if (!is.null(types) && length(types) > 0) {
        
        for (col in names(types)) {
          selectedType <- input[[paste0("type_", col)]]
          if (selectedType == "Date") {
            data[[col]] <- as.Date(data[[col]])
          } else if (selectedType == "Character") {
            data[[col]] <- as.character(data[[col]])
          } else if (selectedType == "Numeric") {
            data[[col]] <- as.numeric(data[[col]])
          } else if (selectedType == "Factor") {
            data[[col]] <- as.factor(data[[col]])
          } else if (selectedType == "Integer") {
            data[[col]] <- as.integer(data[[col]])
          } else if (selectedType == "Logical") {
            data[[col]] <- as.logical(data[[col]])
          }
        }
        output$merged_data_table <- renderDataTable(data,
                                                    options = list(
                                                      pageLength = 10,
                                                      autoWidth = TRUE,
                                                      lengthMenu = list(c(10,50,100, -1), c("10","50","100", "All"))
                                                    ))
      }
      return(data)
    })
    
    #On sélectionne les colonnes à ignorer
    observe({
      if (!is.null(merged_data())) {
        
        
        updateSelectInput(session, "ignore_columns", 
                          choices = colnames(merged_data()), selected = NULL)
        
        output$merged_data_table <- renderDataTable(merged_data()[,setdiff(colnames(merged_data()),input$ignore_columns)],
                                                    options = list(
                                                      pageLength = 10,
                                                      autoWidth = TRUE,
                                                      lengthMenu = list(c(10,50,100, -1), c("10","50","100", "All"))
                                                    ))
      }
    })
    
    
    
    
    #On sélectionne les colonnes numériques
    observe({
      updateSelectInput(session, "var_expliquees", 
                        choices = colnames( merged_data_type() %>% select_if(function(x) is.numeric(x) || is.integer(x))))
      updateSelectInput(session, "var_explicatives", 
                        choices = colnames( merged_data_type() %>% select_if(function(x) is.factor(x))))
      updateSelectInput(session, "emmeans_expliquees", 
                        choices = colnames( merged_data_type() %>% select_if(function(x) is.numeric(x) || is.integer(x))))
      updateSelectInput(session, "emmeans_explicatives", 
                        choices = colnames( merged_data_type() %>% select_if(function(x) is.factor(x))))
      updateSelectInput(session, "var_expliquees_kw", 
                        choices = colnames( merged_data_type() %>% select_if(function(x) is.numeric(x) || is.integer(x))))
      updateSelectInput(session, "var_explicatives_kw", 
                        choices = colnames( merged_data_type() %>% select_if(function(x) is.factor(x))))
      updateSelectInput(session, "khi2_cat", 
                        choices = colnames( merged_data_type() %>% select_if(function(x) is.factor(x))))
      updateSelectInput(session, "khi2_num", 
                        choices = colnames( merged_data_type() %>% select_if(function(x) is.numeric(x) || is.integer(x))))
      updateSelectInput(session, "pearson_num", 
                        choices = colnames( merged_data_type() %>% select_if(function(x) is.numeric(x) || is.integer(x))))
      updateSelectInput(session, "pearson_cat", 
                        choices = colnames( merged_data_type() %>% select_if(function(x) is.factor(x))))
      updateSelectInput(session, "wilcox_num", 
                        choices = colnames( merged_data_type() %>% select_if(function(x) is.numeric(x) || is.integer(x))))
      updateSelectInput(session, "wilcox_cat", 
                        choices = colnames( merged_data_type() %>% select_if(function(x) is.factor(x))))
    })
    
    #On utilise une fonction permettant d'obternir la liste des variables numériques
    num_columns <- eventReactive(input$applyTypes,{
      numeric_integer_columns <- colnames( merged_data_type() %>% select_if(function(x) is.numeric(x) || is.integer(x)))
      return(numeric_integer_columns)
    })
    
    #On utilise une fonction permettant d'obternir la liste des variables cat
    cat_col <- eventReactive(input$applyTypes,{
      return(factor_columns <- sapply(merged_data(), is.factor))
    })
    
    #On utilise une fonction permettant d'obternir la liste des variables
    columns <- eventReactive(input$applyTypes,{
      a <- colnames(merged_data())
      a <- setdiff(a,input$ignore_columns)
      return(a)
    })
    
    
    
    explicative_col <- eventReactive(input$files,{
      return(input$var_expliquees)
    })
    
    kw_explicative_col <- eventReactive(input$kw_button,{
      return(input$var_expliquees_kw)
    })
    
    
    #On utilise une fonction permettant d'obtenir la liste des différentes sources
    liste_source <- eventReactive(input$cat_id,{
      return(unique(merged_data()[[input$cat_id]]))
    })
    
    # Afficher les résultats de la fusion dans une table
    output$merged_data_table <- renderDataTable(merged_data(),
                                                options = list(
                                                  pageLength = 10,
                                                  autoWidth = TRUE,
                                                  lengthMenu = list(c(10,50,100, -1), c("10","50","100", "All"))
                                                )
    )
    
    #=============================================================================
    #Cette partie gère la partie "Graphiques"
    
    observe({
      updateSelectInput(session,"dataset_graph", 
                        choices = c("cars","airquality","iris",input$files$name))
    })
    
    
    graph_data <- eventReactive(input$dataset_graph,{
      if (input$dataset_graph == 'cars') {
        return(datasets::cars)
      }
      if (input$dataset_graph == "airquality"){
        return(datasets::airquality)
      }
      if (input$dataset_graph == "iris"){
        return(datasets::iris)
      }
      if (input$dataset_graph == input$files$name) {
        return(merged_data_type())
      }
    })
    
    observe({
      updateSelectInput(session,"graph_x", choices = colnames(graph_data()))
      updateSelectInput(session,"graph_y", choice = colnames(graph_data()))
      updateSelectInput(session,"graph_fill", choices = c("None",colnames(graph_data())),selected = "None")
      updateSelectInput(session,"facet_a", choices = c(".",colnames(graph_data())),selected = ".")
      updateSelectInput(session,"facet_b", choices = c(".",colnames(graph_data())),selected = ".")
      
    })
    
    
    observe({
      facet_formula = as.formula(paste0(input$facet_a, ' ~ ', input$facet_b))
      print(facet_formula)
    })
    
    observe({
      output$graph_graph <- renderPlotly({
        
        if (input$type_graph == 'Barplot'){
          if (input$graph_fill == 'None'){
            p <- ggplot(data = graph_data(),aes(x=!!sym(input$graph_x),y=!!sym(input$graph_y))) + geom_bar(stat="identity") + facet_grid(as.formula(paste0(input$facet_a, ' ~ ', input$facet_b))) + labs(x = input$graph_x, y = input$graph_y)
            #p <- plot_ly(data = data(),x=~get(input$graph_x),y=~get(input$graph_y),type='bar')
            #p <- p %>% layout(xaxis=list(title=input$graph_x),yaxis=list(title=input$graph_y)) 
            p <- ggplotly(p)
          } else {
            p <-  ggplot(data = graph_data(), aes(x = !!sym(input$graph_x), y = !!sym(input$graph_y), fill = !!sym(input$graph_fill))) +
              geom_bar(stat = "identity") +
              facet_grid((as.formula(paste0(input$facet_a, ' ~ ', input$facet_b)))) +
              labs(x = input$graph_x, y = input$graph_y)
            #p <- plot_ly(data = data(),x=~get(input$graph_x),y=~get(input$graph_y),color=~get(input$graph_fill),type='bar')
            #p <- p %>% layout(xaxis=list(title=input$graph_x),yaxis=list(title=input$graph_y))
            p <- ggplotly(p)
          }
          
          return(p)
        }
        
        if (input$type_graph == 'Geomline'){
          if (input$graph_fill == 'None'){
            p <- ggplot(data = graph_data(),aes(x=!!sym(input$graph_x),y=!!sym(input$graph_y))) +  stat_summary(fun = "mean", geom = "line") + facet_grid(as.formula(paste0(input$facet_a, ' ~ ', input$facet_b))) + labs(x = input$graph_x, y = input$graph_y)
            #p <- plot_ly(data = data(),x=~get(input$graph_x),y=~get(input$graph_y),type='bar')
            #p <- p %>% layout(xaxis=list(title=input$graph_x),yaxis=list(title=input$graph_y)) 
            p <- ggplotly(p)
          } else {
            p <-  ggplot(data = graph_data(), aes(x = !!sym(input$graph_x), y = !!sym(input$graph_y), colour=!!sym(input$graph_fill))) +  stat_summary(fun = "mean", geom = "line",aes(group=!!sym(input$graph_fill)))  +
              facet_grid((as.formula(paste0(input$facet_a, ' ~ ', input$facet_b)))) + labs(x = input$graph_x, y = input$graph_y)
            #p <- plot_ly(data = data(),x=~get(input$graph_x),y=~get(input$graph_y),color=~get(input$graph_fill),type='bar')
            #p <- p %>% layout(xaxis=list(title=input$graph_x),yaxis=list(title=input$graph_y))
            p <- ggplotly(p)
          }
          
          return(p)
        }
        
        if (input$type_graph == 'Boxplot'){
          if (input$graph_fill == 'None'){
            p <-  ggplot(data = graph_data(), aes(x = !!sym(input$graph_x), y = !!sym(input$graph_y))) +
              geom_boxplot() +
              facet_grid((as.formula(paste0(input$facet_a, ' ~ ', input$facet_b)))) +
              labs(x = input$graph_x, y = input$graph_y)
            p <- ggplotly(p)
          } else {
            p <-  ggplot(data = graph_data(), aes(x = !!sym(input$graph_x), y = !!sym(input$graph_y), fill = !!sym(input$graph_fill))) +
              geom_boxplot() +
              facet_grid((as.formula(paste0(input$facet_a, ' ~ ', input$facet_b)))) +
              labs(x = input$graph_x, y = input$graph_y)
          }
          
          return(p)
        }
        if (input$type_graph == 'Density plot'){
          p <- ggplot(graph_data(), aes_string(x=input$graph_x)) + 
            geom_density() + facet_grid((as.formula(paste0(input$facet_a, ' ~ ', input$facet_b))))
          return(ggplotly(p))
        }
        if (input$type_graph == 'Histograme'){
          if (input$graph_fill == 'None') {
            p <- ggplot(graph_data(), aes_string(x=input$graph_x)) + 
              geom_histogram() + facet_grid((as.formula(paste0(input$facet_a, ' ~ ', input$facet_b))))
          } else {
            p <- ggplot(graph_data(), aes_string(x=input$graph_x,color=input$graph_fill)) + 
              geom_histogram() + facet_grid((as.formula(paste0(input$facet_a, ' ~ ', input$facet_b))))
          }
          
          return(ggplotly(p))
        }
        if (input$type_graph == 'Scatterplot'){
          if (input$graph_fill == 'None') {
            p <-  ggplot(data = graph_data(), aes(x = !!sym(input$graph_x), y = !!sym(input$graph_y))) +
              geom_point() +
              facet_grid((as.formula(paste0(input$facet_a, ' ~ ', input$facet_b)))) +
              labs(x = input$graph_x, y = input$graph_y)
            p <- ggplotly(p)
            
          } else {
            p <-  ggplot(data = graph_data(), aes(x = !!sym(input$graph_x), y = !!sym(input$graph_y), fill = !!sym(input$graph_fill))) +
              geom_point() +
              facet_grid((as.formula(paste0(input$facet_a, ' ~ ', input$facet_b)))) +
              labs(x = input$graph_x, y = input$graph_y)
          }
          
          return(p)
        }
        
      })
      
    })
    
    #=============================================================================
    #Cette partie gère l'analyse des données
    
    
    
    shapiro_table <- eventReactive(input$files,{
      shapiro <- shapiro_table_function(data=merged_data_type(),
                                        num_col=num_col())
      
      return(shapiro)
    })
    
    
    output$shapiro_table <- renderDataTable(shapiro_table(),
                                            options = list(
                                              pageLength = 10,
                                              autoWidth = TRUE,
                                              lengthMenu = list(c(10,50,100, -1), c("10","50","100", "All"))
                                            )
    )
    
    
    
    anova_table <- eventReactive(input$anova_button,{
      anova_table <- anova_table_function(data=merged_data_type(),
                                          var_expliquee=input$var_expliquees, 
                                          var_explicatives=input$var_explicatives,
                                          output=output)
      return(anova_table)
    })
    
    
    
    output$anova_table <- renderDataTable(anova_table(),
                                          extensions = "Buttons",
                                          options = list(
                                            pageLength = 10,
                                            autoWidth = TRUE,
                                            lengthMenu = list(c(10,50,100, -1), c("10","50","100", "All"))
                                          )
    )
    
    output$anova_button_download <- downloadHandler(
      filename = function(){"anova_results.csv"}, 
      content = function(fname){
        write.csv(anova_table(), fname)
      }
    )
    
    
    Temoins <- reactiveVal(list())
    
    observe({
      temoins <- list()
      for (col in input$emmeans_explicatives) {
        temoins[[col]] <- selectInput(
          inputId = paste0("temoin_", col),
          label = paste0("temoin : ", col),
          choices = unique(merged_data()[[col]]),
          selected = unique(merged_data()[[col]])[1]
        )
      }
      Temoins(temoins)
    })
    
    output$Temoins <- renderUI({
      temoins <- Temoins()
      if (!is.null(temoins) && length(temoins) > 0) {
        div(
          lapply(names(temoins), function(col) {
            temoins[[col]]
          })
        )
      }
    })
    
    
    ref_index <- eventReactive(input$emmeans_button,{
      
      ref_index <- ref_index_function(data=merged_data_type(),
                                      var_expliquee=input$emmeans_expliquees,
                                      var_explicatives=input$emmeans_explicatives,
                                      input=input,
                                      Temoins=Temoins())
      
      return(ref_index)
      
    })
    
    emmeans_table <- eventReactive(input$emmeans_button,{
      
      emmeans_table <- emmeans_table_function(data=merged_data_type(),
                             var_expliquee=input$emmeans_expliquees,
                             var_explicatives=input$emmeans_explicatives,
                             type_comp=input$type_comp,
                             ref_index = ref_index(),
                             output=output)
      return(emmeans_table)
      
      
    })
    
    
    
    
    contrasts_table <- eventReactive(input$emmeans_button,{
    contrasts_table <- contrasts_table_function(data=merged_data_type(),
                                                var_expliquee=input$emmeans_expliquees,
                                                var_explicatives=input$emmeans_explicatives,
                                                type_comp=input$type_comp,
                                                ref_index = ref_index(),
                                                output=output)
    
    return(contrasts_table)
      
    })
    
    
    
    output$emmeans_table <- renderDataTable(datatable(emmeans_table(),
                                                      options = list(
                                                        pageLength = 10,
                                                        autoWidth = TRUE,
                                                        lengthMenu = list(c(10,50,100, -1), c("10","50","100", "All"))
                                                      ),
                                                      caption = "Moyennes ajustées",
    )
    )
    
    
    output$emmeans_button_download <- downloadHandler(
      filename = function(){"emmeans_results.csv"}, 
      content = function(fname){
        write.csv(emmeans_table(), fname,row.names = FALSE)
      }
    )
    
    output$contrast_table <- renderDataTable(datatable(contrasts_table(),
                                                       options = list(
                                                         pageLength = 10,
                                                         autoWidth = TRUE,
                                                         lengthMenu = list(c(10,50,100, -1), c("10","50","100", "All"))
                                                       ),
                                                       caption = "Contrasts",
                                                      )
                                                )
    
    output$contrast_button_download <- downloadHandler(
      filename = function(){"contrast_results.csv"}, 
      content = function(fname){
        write.csv(contrasts_table(), fname,row.names = FALSE)
      }
    )
    
    kw_table <- eventReactive(input$kw_button,{
      
      kw_table <- kw_table_function(cat_col=inputvar_explicatives_kw,
                                    num_col=input$var_expliquees_kw,
                                    data=merged_data_type(),
                                    output=output)
      
      return(kw_table)
      
    })
    
    output$kw_table <- renderDataTable(kw_table(),
                                       options = list(
                                         pageLength = 10,
                                         autoWidth = TRUE,
                                         lengthMenu = list(c(10,50,100, -1), c("10","50","100", "All"))
                                       )
    )
    
    
    
    pearson_result <- eventReactive(input$pearson_button,{
      X_level = unique(merged_data()[[input$pearson_cat]])[1]
      Y_level = unique(merged_data()[[input$pearson_cat]])[2]
      X = merged_data()[merged_data()[,input$pearson_cat]==X_level,input$pearson_num]
      Y = merged_data()[merged_data()[,input$pearson_cat]==Y_level,input$pearson_num]
      x = pull(X,1)
      y = pull(Y,1)
      print(cor.test(x,y, method = "pearson"))
    })
    
    output$pearson_result <- renderPrint(pearson_result())
    
    
    wilcox_result <- eventReactive(input$wilcox_button,{
      X_level = unique(merged_data()[[input$wilcox_cat]])[1]
      Y_level = unique(merged_data()[[input$wilcox_cat]])[2]
      X = merged_data()[merged_data()[,input$wilcox_cat]==X_level,input$wilcox_num]
      Y = merged_data()[merged_data()[,input$wilcox_cat]==Y_level,input$wilcox_num]
      x = pull(X,1)
      y = pull(Y,1)
      print(wilcox.test(x,y, paired=TRUE))
    })
    
    output$wilcox_result <- renderPrint(wilcox_result())
    #=============================================================================
    #Cette partie gère la partie "Modèles classiques"
    observe({
      updateSelectInput(session, "acp_col", choices = num_columns())
    })
    
    observe({
      updateSliderInput(session,"pls_slider",max = length(input$pls_x))
    })
    
    observeEvent(input$acp_button,{
      output$plotly_acp_graph <- renderPlotly({
        acp_data <- merged_data()[, input$acp_col]
        res.pca <- prcomp(acp_data, scale = TRUE)
        pl <- fviz_pca_ind(
          res.pca,
          col.ind = merged_data()[["Source"]],  # Color by the quality of representation
          repel = TRUE,  # Avoid text overlapping
          label = "None"
        ) + scale_color_discrete()
        
        # Convertir le graphique ggplot2 en graphique plotly
        pl <- ggplotly(pl)
        
        return(pl)
      })
      
    })
    
    observe({
      updateSelectInput(session, "pls_x", choices = num_columns())
      updateSelectInput(session, "pls_y", choices = num_columns())
    })
    
    
    observeEvent(input$pls_button,{
      output$pls_text <-  renderPrint({
        pls_formula <- as.formula(paste0(input$pls_y,'~',paste(input$pls_x,collapse='*')))
        pls.result <- plsr(pls_formula,data=merged_data_type(),scale=TRUE, validation="CV")
        pls_sum <- summary(pls.result)
        return(pls_sum)
      })
      
    })
    
    observe({
      updateSelectInput(session, "lm_x", choices = num_columns())
      updateSelectInput(session, "lm_y", choices = colnames( merged_data_type() %>% select_if(function(x) is.factor(x))))
    })
    
    
    observeEvent(input$lm_button,{
      output$lm_text <-  renderPrint({
        merged_data <- merged_data_type()
        lm_formula <- as.formula(paste0(input$lm_x,'~',paste(input$lm_y,collapse='*')))
        lm_result <- lm(lm_formula,data = merged_data)
        return(summary(lm_result))
      })
      
    })
    
    
    #output$pls_text <-  renderText({'print'})
    
    
  })
}