ui <- fluidPage(
  useShinyjs(),
  
  
  # Ici on empêche l'utilisateur de pouvoir rafraîchir la page accidentellement  
  tags$script('
    window.onbeforeunload = function() {
      return "Voulez-vous vraiment quitter cette page ?";
    };
  '),
  
  
  # Ici on a créé une fonction permettant de masquer tous les éléments de la page
  extendShinyjs(text = "
    shinyjs.hideAllElements = function() {
      $('body > *').hide(); 
    }
  ",functions="shinyjs.hideAllElements"),
  
  
  #theme = shinytheme("readable"),
  navbarPage(
    "Stat Plateform",
    tabPanel("Accueil",
             fluidRow(
               column(4,
                      textOutput("welcome"),
                      wellPanel(
                        fileInput("files", "Sélectionnez les fichiers Excel (xlsx) :", multiple = TRUE),
                        selectInput("ignore_columns", "Colonnes à ignorer :", multiple = TRUE, choices = NULL),
                        bsTooltip("columnTypes","Choisir les types des colonnes",placement = "right",trigger = "hover"),
                        uiOutput("columnTypes"),
                        actionButton("applyTypes", "Appliquer les types de données"),
                      )
               ),
               column(8,
                      wellPanel(
                        style = 'overflow-x: scroll;height: 950px;',
                        DT::dataTableOutput("merged_data_table")
                      )
               )
             ),
    ),
    navbarMenu("Analyse",
               tabPanel("ANOVA",
                        fluidRow(
                          column(4,
                                 wellPanel(
                                   selectInput("var_expliquees","Choisir la variable à expliquer",multiple = FALSE,choices = NULL),
                                   selectInput("var_explicatives","Choisir les variables explicatives",multiple = TRUE,choices = NULL),
                                   actionButton("anova_button", "ANOVA",class="btn-primary")
                                 ),
                                 wellPanel(
                                   selectInput("emmeans_expliquees","Choisir la variable à expliquer",multiple = FALSE,choices = NULL),
                                   selectInput("emmeans_explicatives","Choisir les variables explicatives",multiple = TRUE,choices = NULL),
                                   uiOutput("Temoins"),
                                   selectInput("type_comp", "Type de comparaison", multiple = FALSE, choices = c("pairwise","trt.vs.ctrl")),
                                   actionButton("emmeans_button", "EMMEANS",class="btn-primary")
                                 )
                          ),
                          column(8,
                                 tabsetPanel(
                                   tabPanel('Table',
                                            wellPanel(
                                              style = 'overflow-x: scroll;height: 650px;',
                                              DT::dataTableOutput("anova_table"),
                                              downloadButton("anova_button_download", "Télécharger le tableau",class="btn-primary")
                                            ),
                                            wellPanel(
                                              style = 'overflow-x: scroll;height: 650px;',
                                              DT::dataTableOutput("emmeans_table"),
                                              downloadButton("emmeans_button_download", "Télécharger le tableau",class="btn-primary"),
                                              DT::dataTableOutput("contrast_table"),
                                              downloadButton("contrast_button_download", "Télécharger le tableau",class="btn-primary")
                                            )
                                   ),
                                   tabPanel("result",
                                            h2("Résultat Anova"),
                                            verbatimTextOutput("anova_result"),
                                            h2("Résultat Emmeans"),
                                            verbatimTextOutput("emmeans_result"), 
                                            h2('Moyenne comparée'),
                                            plotlyOutput('graph_compared_mean')
                                   )
                                 )
                          )
                        )
               ),
               tabPanel("Test de Krustal-Wallis",
                        fluidRow(
                          column(4,
                                 wellPanel(
                                   selectInput("var_expliquees_kw","Choisir la variable à expliquer",multiple = FALSE,choices = NULL),
                                   selectInput("var_explicatives_kw","Choisir la variable explicative",multiple = FALSE,choices = NULL),
                                   actionButton("kw_button", "KW",class="btn-primary")
                                 ),
                          ),
                          column(8,
                                 wellPanel(
                                   style = 'overflow-x: scroll;height: 650px;',
                                   DT::dataTableOutput("kw_table"),
                                   downloadButton("kw_button_download", "Télécharger le tableau",class="btn-primary")
                                 ),
                                 wellPanel(
                                   "KW Resultat",
                                   verbatimTextOutput("kw_resultat")
                                 )
                          )
                        )
               ),
               tabPanel("Khi 2",
                        fluidRow(
                          column(4,
                                 wellPanel(
                                   selectizeInput("khi2_cat","Choisir les variables",multiple = TRUE,choices = NULL),
                                   actionButton("khi2_button", "KHI2",class="btn-primary")
                                 )
                          ),
                          column(8,
                                 wellPanel(
                                   style = 'overflow-x: scroll;height: 650px;',
                                   DT::dataTableOutput("khi2_table"),
                                   downloadButton("khi2_button_download", "Télécharger le tableau",class="btn-primary")
                                 )
                          )
                        )
               ), 
               tabPanel("Test de Pearson",
                        fluidRow(
                          column(4,
                                 wellPanel(
                                   selectizeInput("pearson_num","Choisir la variable quantitative",multiple = FALSE,choices = NULL),
                                   selectizeInput("pearson_cat","Choisir la variable catégorielle",multiple = FALSE,choices = NULL),
                                   actionButton("pearson_button", "Pearson",class="btn-primary")
                                 ),
                          ),
                          column(8,
                                 wellPanel("Pearson Resultat",
                                           verbatimTextOutput("pearson_result")
                                 )
                          )
                        )
               ),
               tabPanel("Test de Wilcoxon",
                        fluidRow(
                          column(4,
                                 wellPanel(
                                   selectizeInput("wilcox_num","Choisir la variable quantitative",multiple = FALSE,choices = NULL),
                                   selectizeInput("wilcox_cat","Choisir la variable catégorielle",multiple = FALSE,choices = NULL),
                                   actionButton("wilcox_button", "Wilcoxon",class="btn-primary")
                                 ),
                          ),
                          column(8,
                                 wellPanel("Wilcoxon Resultat",
                                           verbatimTextOutput("wilcox_result")
                                 )
                          )
                        )
               ),
    ),
    tabPanel("Graphique",
             column(3,
                    wellPanel(
                      h2('Analyse Statistique'),
                      selectInput("dataset_graph", "dataset_graph à prendre", multiple = FALSE, choices = c("cars","airquality","iris")),
                      selectInput("type_graph", "Type de graphique", multiple = FALSE, choices = c("Barplot","Boxplot","Density plot","Geomline","Histograme","Scatterplot")),
                      selectInput("graph_x", "X", multiple = FALSE, choices = NULL),
                      selectInput("graph_y", "Y", multiple = FALSE, choices = NULL),
                      selectInput("graph_fill", "Fill", multiple = FALSE, choices = NULL),
                      selectInput("facet_a","Face grid",multiple = FALSE,choices = '.',),
                      selectInput("facet_b","Face grid",multiple = FALSE,choices = '.',)
                    )
             ), 
             column(9,
                    fluidRow(
                      wellPanel(
                        plotlyOutput("graph_graph"),
                      )
                    )
             )
    ),
    tabPanel("Instructions", 
             fluidRow(
               column(8,
                      includeMarkdown("README.md")
               )
             )
    ), 
    navbarMenu("Régressions",
               tabPanel("ACP",
                        column(3,
                               wellPanel(
                                 h2('ACP'),
                                 selectInput("acp_col", "Numerical Columns", multiple = TRUE, choices = NULL),
                                 actionButton("acp_button", "ACP",class="btn-primary"),
                               ),
                        ), 
                        column(9,
                               fluidRow(
                                 wellPanel(
                                   plotlyOutput("plotly_acp_graph")
                                 )
                               )
                        )
               ),
               tabPanel("PLS",
                        column(3,
                               wellPanel(
                                 h2('PLS'),
                                 selectInput("pls_x", "X", multiple = TRUE, choices = NULL),
                                 selectInput("pls_y", "Y", multiple = FALSE, choices = NULL),
                                 sliderInput("pls_slider","Nombre de composante",1,1,1,step = 1),
                                 actionButton("pls_button", "PLS",class="btn-primary")
                               )
                        ), 
                        column(9,
                               fluidRow(
                                 wellPanel( 
                                   h2('Résultats'),
                                   verbatimTextOutput("pls_text"),
                                 )
                               )
                        )
               ),
               tabPanel("Linéaire",
                        column(6,
                               wellPanel(
                                 h2('Régression Linéaire'),
                                 selectInput("lm_x", "X", multiple = FALSE, choices = NULL),
                                 selectInput("lm_y", "Y", multiple = TRUE, choices = NULL),
                                 actionButton("lm_button", "Régression",class="btn-primary")
                               )
                        ), 
                        column(6,
                               fluidRow(
                                 wellPanel( 
                                   h2('Résultats'),
                                   verbatimTextOutput("lm_text"),
                                 )
                               )
                        )
               ),
    ),
  )
)
